// C-BeerControl 
// Date:    2/19/2010
// Author:  230 / Coach

// Status: verified operational with:
//          * Freescale HC12 Evaluation board
//          * PC Beer Model 

// Objective: to demonstrate simple process control



// I/O Interface :

// Input from from Beer Model:
// ==========================
//	bit 0: Run / Stop
//  bit 1: not used
//	bit 2: Temp Limit
//	bit 3: Level

// Output to Beer Model:
// ========================
//	bit 4: Pump
//	bit 5: Inlet Valve
//	bit 6: Heater
//	bit 7: Chiller 


#include <hidef.h>         /* common defines and macros */
#include "device_headers.h" /*Chip specific defines and macros*/


/* On-board interfaced components */
void AppMod_PB_Init(void);
void AppMod_LED_Init(void);

/*************************************************************************/
/* Initialization routine to configure PB0 to PB3 pins as inputs. */
void AppMod_PB_Init(void)              
{
  DDRB_BIT0 = 0;
  DDRB_BIT1 = 0;
  DDRB_BIT2 = 0;
  DDRB_BIT3 = 0;

}
/************************************************************************************************/

/* Initialization routine to configure PB4 to PB7 outputs. */
void AppMod_LED_Init(void)
{

  DDRB_BIT4 = 1;
  DDRB_BIT5 = 1;
  DDRB_BIT6 = 1;
  DDRB_BIT7 = 1;
  
}
/**********************************************************************************/



void main(void) {

  int input = 0;
  int output;
  long count=0;
  long count2=0; 
    
  int pump = 0x10;  //bit 4
  int pump_activeLo = 0xEF;    //Valve:bit 5 
        //bit7 6 5 4  3 2 1 0
        //   1 1 1 0  1 1 1 1  : 0xEF
  
  int valve = 0x20; //bit 5   
  int valve_activeLo = 0xDF;    //Valve:bit 5 
        //bit7 6 5 4  3 2 1 0
        //   1 1 0 1  1 1 1 1  : 0xDF
        
  int heater = 0x40; //bit 6  
  int heater_activeLo = 0xBF;
        //bit7 6 5 4  3 2 1 0
        //   1 0 1 1  1 1 1 1  : 0xBF  
  	
  int chiller = 0x80; //bit 7
  int chiller_activeLo = 0x7F;
        //bit7 6 5 4  3 2 1 0
        //   0 1 1 1  1 1 1 1  : 0x7F  
  
  
  int PB0=0;
  int PB1=0;
  int PB2=0;
  int PB3=0;
 
	int state = 0;

	int toggle =0;

	char control='0';
	int run = 0;
  int Level= 0;
	int	Temp = 0;
  
  INIT_SLK_PBs;  // Initialize PB inputs
  
  INIT_SLK_LEDs; // Initialize PB outputs
  
  output = 0xF0;   // turn off all outputs, active low
  PORTB = output;   // turn off all outputs, active low

 while (1){    //infinite loop
  
 
 input = PORTB;         // Read inputs
 run = input & 0x01;
 Level = input & 0x08;  //bit 3
 Temp = input & 0x04;   //bit 2
 
// Sequential Control (state machine) starts here:

	switch (state)
	{
	case 0:				// Do nothing, waiting for Run request
	  if (run !=0) {
	    state = 1;
	  }
		break;
		
	case 1:
		// Turn on Inlet Valve:
		output = output & valve_activeLo  ;
      	PORTB = output;
		state = 2;
		break;
		
	case 2:
		if (Level !=0)
			{
			output = output | valve;			//Turn OFF Inlet Valve, active Lo
				PORTB = output;
				state = 3;
			}
		break;
		
	case 3:
		//Turn ON Heater
		output = output & heater_activeLo ;	
		  	PORTB = output;
		state = 4;
		break;
		
	case 4:
		if (Temp !=0)
			{
		//Tyrn OFF Heater
			output = output | heater;			//Turn OFF heater, active Lo
		  	PORTB = output; 		
				state = 5;
			}
		break;
		
	case 5: 
	  // 7 seconds Time Delay
	  for (count =0; count <500; count++) 
	  {
	   for (count2 =0; count2 <1000; count2++) {  }
	  }
	     state = 6;
	     break;
	     
	case 6:
		 // Chill it
		output = output & chiller_activeLo ;	
		  	PORTB = output; 		 
		state = 7;
		break;
				
	case 7:
		if (Temp == 0)
			{
		// turn OFF chiller
			output = output | chiller;			//Turn OFF chiller, active Lo
		 	PORTB = output;
		  // PORTB = 0x80;     //testing only :force chiller off
			state = 8;
			}
		break;
		
	case 8:
	 // Pump it
		output = output & pump_activeLo ;	
		  	PORTB = output;
		state = 9;
		break;
		
	case 9:
		if (Level ==0)
			{
			  // turn OFF Pump
			output = output | pump;			//Turn OFF pump, active Lo
		 	PORTB = output; 			  
			  
				state = 0;
			}
		break;

	} // end switch (state)
	
 	
 		
	}// end while(1)    
   
  
}   // end Main


