// C-BeerControl 
// Date:    2/19/2010
// Author:  230 / Coach

// Status: starting...

// Objective:
//  * Provide template for easy start to experiment with C & Freescale microcontroller

// Task:

// *** Complete Beer Pasteurization Process Control based on prescribed functional requirements

// I/O Interface :

// Input from from Beer Model:
// ==========================
//	bit 0: Run / Stop   :  0: Stop, 1: Run
//  bit 1: not used
//	bit 2: Temp Limit   :  0: Cold, 1: Hot
//	bit 3: Level        :  0: empty, 1: full

// Output to Beer Model:
// ========================
//	bit 4: Pump           : 0: ON, 1:OFF   (active Low)
//	bit 5: Inlet Valve    : 0: ON, 1:OFF   (active Low)
//	bit 6: Heater         : 0: ON, 1:OFF   (active Low)
//	bit 7: Chiller        : 0: ON, 1:OFF   (active Low)

#include <hidef.h>         /* common defines and macros */
#include "device_headers.h" /*Chip specific defines and macros*/ 

/* On-board interfaced components */
void AppMod_PB_Init(void);
void AppMod_LED_Init(void);

/************************************************************************/
/* Initialization routine to configure PB0 to PB3 pins as inputs. */
void AppMod_PB_Init(void)              
{
  DDRB_BIT0 = 0;
  DDRB_BIT1 = 0;
  DDRB_BIT2 = 0;
  DDRB_BIT3 = 0;

}
/***********************************************************************/

/* Initialization routine to configure PB4 to PB7 outputs. */
void AppMod_LED_Init(void)
{

  DDRB_BIT4 = 1;
  DDRB_BIT5 = 1;
  DDRB_BIT6 = 1;
  DDRB_BIT7 = 1;
  
}
/************************************************************/  

void main(void) {

  int input = 0;
  int output;   
  int pump = 0x10;  //bit 4
  int valve = 0x20; //bit 5
  int valve_activeLo = 0xDF;    //Valve:bit 5 
        //bit7 6 5 4  3 2 1 0
        //   1 1 0 1  1 1 1 1  : 0xDF
  int heater = 0x40; //bit 6
  int chiller = 0x80; //bit 7
  int PB0=0;
  int PB1=0;
  int PB2=0;
  int PB3=0;
 
	int state = 0;

	int toggle =0;

	char control='0';
	int run = 0;
  int Level= 0;
	int	Temp = 0;
  
  INIT_SLK_PBs;  // Initialize PB inputs
  
  INIT_SLK_LEDs; // Initialize PB outputs
  
  output = 0xFF;   // turn off all outputs, active low
  PORTB = 0xF0;   // turn off all outputs, active low

 while (1){    //infinite loop
  
 
 input = PORTB;         // Read inputs
 run = input & 0x01;
 Level = input & 0x08;  //bit 3
 Temp = input & 0x04;   //bit 2
 
// Sequential Control (state machine) starts here:

	switch (state)
	{
	case 0:				// Do nothing, waiting for Run request
	  if (run !=0) {
	    state = 1;
	  }
		break;
		
	case 1:
		// Turn on Inlet Valve:
		output = output & valve_activeLo  ;

		state = 2;
		break;
		
	case 2:
		if (Level !=0)
			{
			output = output | valve;			//Turn OFF Inlet Valve, active Lo
				state = 3;
			}
		break;
		
	case 3:
		//Turn ON Heater
		state = 4;
		break;
		
	case 4:
		if (Temp !=0)
			{
			 //Tyrn OFF Heater
				state = 6;
			}
		break;
		
	case 5: break;
	
	case 6:
		 // Chill it
		state = 7;
		break;
				
	case 7:
		if (Temp ==0)
			{
			 // turn OFF chiller
				state = 8;
			}
		break;
		
	case 8:
	 // Pump it
		state = 9;
		break;
		
	case 9:
		if (Level ==0)
			{
			  // turn OFF Pump
				state = 0;
			}
		break;

	} // end switch (state)
	
 		PORTB = output;
 		
	}// end while(1)    
   
  
}   // end Main


